<?php

namespace Drupal\fox\Plugin\FoxCommand;

use Drupal\Component\Serialization\Json;

/**
 * BROWSE fox command.
 *
 * @FoxCommand(
 *   id = "browse",
 *   label = @Translation("Browse fields for context record. Usage: BROWSE [fields]")
 * )
 */
class FoxCommandBrowse extends FoxCommandBaseClass {

  /**
   * {@inheritdoc}
   */
  public function execute(array $params, array $variables): array {
    $helper = $this->foxCommandsHelper();

    $entity = $helper->getEntity($variables);
    if (is_array($entity) and isset($entity['error'])) {
      return $entity;
    }
    if (empty($entity)) {
      return $this->errorReturn($this->t('Empty entity'));
    }

    if (method_exists($entity, 'getFields')) {
      $fieldsList = $entity->getFields();
    }
    else {
      if (empty($params)) {
        return $this->errorReturn($this->t('Need to set fields for the entity type'));
      }
    }

    if (empty($params)) {
      $fields = array_keys($fieldsList);
    }
    else {
      $fields = $helper->prepareParameters($params);
    }

    $data = $row = $header = [];
    foreach ($fields as $field) {
      $name = trim($field);
      if ((method_exists($entity, 'hasField') && !$entity->hasField($name))) {
        return $this->errorReturn($this->t('Wrong field "@name"', [
          '@name' => $name,
        ]));
      }

      $value = $entity->get($name);
      if ($value && !is_string($value)) {
        $value = $value->getValue();
      }

      $row[] = Json::encode($value);

      if (isset($fieldsList)) {
        $definition = $fieldsList[$name]->getFieldDefinition();
        $header[] = $definition->getLabel() . ' (' . $definition->getName() . ')';
      }
      else {
        $header[] = $name;
      }
    }
    $data[] = $row;

    return [
      'message' => [
        'header' => $header,
        'data' => $data,
        'horizontal' => TRUE,
      ],
    ];
  }

}
